import { enablePatches, produce, applyPatches } from 'immer';
import { get, set } from '@ember/object';
import { tracked } from '@glimmer/tracking';

function _initializerDefineProperty(target, property, descriptor, context) {
  if (!descriptor) return;
  Object.defineProperty(target, property, {
    enumerable: descriptor.enumerable,
    configurable: descriptor.configurable,
    writable: descriptor.writable,
    value: descriptor.initializer ? descriptor.initializer.call(context) : void 0
  });
}

function _typeof(o) {
  "@babel/helpers - typeof";

  return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) {
    return typeof o;
  } : function (o) {
    return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o;
  }, _typeof(o);
}

function _toPrimitive(input, hint) {
  if (_typeof(input) !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];
  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (_typeof(res) !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (hint === "string" ? String : Number)(input);
}

function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");
  return _typeof(key) === "symbol" ? key : String(key);
}

function _defineProperty(obj, key, value) {
  key = _toPropertyKey(key);
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }
  return obj;
}

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) {
  var desc = {};
  Object.keys(descriptor).forEach(function (key) {
    desc[key] = descriptor[key];
  });
  desc.enumerable = !!desc.enumerable;
  desc.configurable = !!desc.configurable;
  if ('value' in desc || desc.initializer) {
    desc.writable = true;
  }
  desc = decorators.slice().reverse().reduce(function (desc, decorator) {
    return decorator(target, property, desc) || desc;
  }, desc);
  if (context && desc.initializer !== void 0) {
    desc.value = desc.initializer ? desc.initializer.call(context) : void 0;
    desc.initializer = undefined;
  }
  if (desc.initializer === void 0) {
    Object.defineProperty(target, property, desc);
    desc = null;
  }
  return desc;
}

function aggregatedLastChanges(arr) {
  const result = [];
  arr.forEach(item => {
    const existingIndex = result.findIndex(e => e.key === item.key);
    if (existingIndex !== -1) {
      result[existingIndex] = item;
    } else {
      result.push(item);
    }
  });
  return result;
}

class ChangesetEventEmitter {
  constructor(changeset) {
    _defineProperty(this, "events", {
      set: new Map()
    });
    this.changeset = changeset;
  }
  emitOnSet(key, value) {
    this.events['set'].forEach(callback => callback(key, value, this.changeset));
  }
  on(event, callback) {
    const uniqueId = Symbol();
    this.events[event].set(uniqueId, callback);
    return uniqueId;
  }
  off(event, uniqueId) {
    this.events[event].delete(uniqueId);
  }
}

var _class, _descriptor, _descriptor2, _descriptor3;
enablePatches();
let ImmerChangeset = (_class = class ImmerChangeset {
  get changes() {
    return aggregatedLastChanges(this.normalizedPatches());
  }
  get errors() {
    return Object.values(this.innerErrors);
  }
  get isValid() {
    return this.errors.length === 0;
  }
  get isPristine() {
    return this.patches.length + this.inversePatches.length === 0;
  }
  get isInvalid() {
    return !this.isValid;
  }
  get isDirty() {
    return !this.isPristine;
  }
  constructor(data) {
    _initializerDefineProperty(this, "data", _descriptor, this);
    _initializerDefineProperty(this, "draftData", _descriptor2, this);
    _initializerDefineProperty(this, "innerErrors", _descriptor3, this);
    _defineProperty(this, "patches", []);
    _defineProperty(this, "inversePatches", []);
    _defineProperty(this, "eventEmitter", new ChangesetEventEmitter(this));
    this.data = produce(data, () => {});
    this.draftData = produce(data, () => {});
  }
  execute() {
    this.data = applyPatches(this.data, this.patches);
  }
  unexecute() {
    this.data = applyPatches(this.data, this.inversePatches);
  }
  save() {
    this.data = applyPatches(this.data, this.patches);
    this.resetPatches();
  }
  rollback() {
    this.draftData = applyPatches(this.draftData, this.inversePatches);
  }
  rollbackProperty(property) {
    this.set(property, get(this.data, property));
  }
  addError(key, error) {
    this.innerErrors = {
      ...this.innerErrors,
      [key]: error
    };
  }
  removeError(key) {
    delete this.innerErrors[key];
    this.innerErrors = {
      ...this.innerErrors
    };
  }
  get(key) {
    return get(this.draftData, key);
  }
  set(key, value) {
    this.draftData = produce(this.draftData, d => {
      set(d, key, value);
    }, (patches, inversePatches) => {
      this.patches.push(...patches);
      this.inversePatches.push(...inversePatches);
    });
    this.eventEmitter.emitOnSet(key, value);
  }
  async validate(validation) {
    const errors = await validation(this.draftData);
    this.innerErrors = errors.reduce((p, c) => {
      p[c.key] = c;
      return p;
    }, {});
  }
  normalizedPatches() {
    return this.patches.map(patch => ({
      value: patch.value,
      key: patch.path.join('.')
    }));
  }
  resetPatches() {
    this.patches = [];
    this.inversePatches = [];
  }
  onSet(fn) {
    const listener = this.eventEmitter.on('set', fn);
    return () => {
      this.eventEmitter.off('set', listener);
    };
  }
}, (_descriptor = _applyDecoratedDescriptor(_class.prototype, "data", [tracked], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor2 = _applyDecoratedDescriptor(_class.prototype, "draftData", [tracked], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor3 = _applyDecoratedDescriptor(_class.prototype, "innerErrors", [tracked], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: function () {
    return {};
  }
})), _class);

export { ImmerChangeset as I, _applyDecoratedDescriptor as _ };
//# sourceMappingURL=immer-changeset-36d7e0a3.js.map
